/*
 * Copyright 2020 IQTIG – Institut für Qualitätssicherung und Transparenz im Gesundheitswesen.
 * Dieser Code ist urheberrechtlich geschützt (Copyright).
 * Das Urheberrecht liegt, soweit nicht ausdrücklich anders gekennzeichnet, beim IQTIG.
 * Wer gegen das Urheberrecht verstößt, macht sich gem. § 106 ff Urhebergesetz strafbar.
 * Er wird zudem kostenpflichtig abgemahnt und muss Schadensersatz leisten.
 */
package org.iqtig.od.psp.impl.examples;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import org.iqtig.od.psp.api.KeyManagement;
import org.iqtig.od.psp.api.beans.CertificateCreationTO;
import org.iqtig.od.psp.api.beans.CertificateIdentity;
import org.iqtig.od.psp.api.enums.DataCollectionPoint;
import org.iqtig.od.psp.api.enums.Region;
import org.iqtig.od.psp.api.exception.PspApiException;
import org.iqtig.od.psp.impl.APIBuilder;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

/**
 * Beispiel für den Umgang mit Schlüsseln / Zertifikaten
 */
class KeyManagementExampleTest {

	private static final String TEST_DIRECTORY = "target/testing/all-certificates";

	private static final LocalDate TEST_START_DATE = LocalDate.of(2020, 1, 1);
	private static final LocalDate TEST_END_DATE = LocalDate.of(2030, 6, 30);
	private static final String TEST_PASSWORD = "test";

	private KeyManagement keyManagement;

	@BeforeAll
	static void beforeAll() throws IOException {
		final Path path = Path.of(TEST_DIRECTORY);
		if (Files.notExists(path)) {
			Files.createDirectories(path);
		}
	}

	@BeforeEach
	void beforeEach() {
		final APIBuilder builder = new APIBuilder();
		keyManagement = Assertions.assertDoesNotThrow(builder::getKeyManagement);
	}

	/**
	 * Exemplarisches Erstellen eines Zertifikats
	 */
	@Test
	void test() throws PspApiException, IOException {
		// Arrange
		final CertificateIdentity certIdentity = new CertificateIdentity(DataCollectionPoint.LQS, Region.BB);
		final String startDateString = TEST_START_DATE.format(DateTimeFormatter.ofPattern("yyyyMMdd"));
		final String baseFileName = DataCollectionPoint.LQS.name() + "$$" + Region.BB.name() + startDateString;
		final Path keyStorePath = Path.of(TEST_DIRECTORY, baseFileName + ".p12");
		final Path certificatePath = Path.of(TEST_DIRECTORY, baseFileName + ".cer");
		// Act & Assert
		createCertificate(certIdentity, keyStorePath);
		extractCertificate(keyStorePath, certificatePath);
	}

	private void extractCertificate(final Path keyStorePath, final Path certificatePath) throws IOException, PspApiException {
		// Act
		try (
			final InputStream keyStoreInputStream = Files.newInputStream(keyStorePath);
			final OutputStream certificateOutputStream = new FileOutputStream(certificatePath.toFile())
		) {
			keyManagement.extractAndStoreX509CertificateFromKeyStore(keyStoreInputStream, TEST_PASSWORD, certificateOutputStream);
		}
		Assertions.assertTrue(Files.exists(certificatePath));
	}

	private void createCertificate(final CertificateIdentity certIdentity, final Path keyStorePath) throws IOException, PspApiException {
		// Act
		try (final OutputStream keyStoreOutputStream = new FileOutputStream(keyStorePath.toFile())) {
			keyManagement.generateCertificateWithKeyPairAndStoreToKeyStore(new CertificateCreationTO(certIdentity, TEST_START_DATE, TEST_END_DATE),
				TEST_PASSWORD, keyStoreOutputStream);
		}
		Assertions.assertTrue(Files.exists(keyStorePath));
	}
}
